@@ -23,10 +23,11 @@ import logging
 import os
 import subprocess
 import typing as t
+from pathlib import Path
 
 import databind.core.dataclasses as dataclasses
 import docspec
-import nr.fs
+from nr.util.git import Git, NoCurrentBranchError
 
 from pydoc_markdown.interfaces import Context, SourceLinker
 
@@ -43,6 +44,19 @@ def _getoutput(cmd: t.List[str], cwd: str = None) -> str:
   return stdout
 
 
+def git_get_toplevel(cwd: str) -> str:
+  return _getoutput(['git', 'rev-parse', '--show-toplevel'], cwd=cwd).strip()
+
+
+def git_get_current_commit_sha(cwd: str) -> str:
+  return _getoutput(['git', 'rev-parse', 'HEAD'], cwd=cwd).strip()
+
+
+def git_get_current_branch(cwd: str) -> str:
+  branches = _getoutput(['git', 'branch'], cwd=cwd).strip().splitlines()
+  return next(b for b in branches if b.startswith('*')).split()[1]
+
+
 @dataclasses.dataclass
 class BaseGitSourceLinker(SourceLinker):
   """
@@ -55,6 +69,9 @@ class BaseGitSourceLinker(SourceLinker):
   #: the context directory.
   root: t.Optional[str] = None
 
+  #: Use the branch name instead of the current SHA to generate URLs.
+  use_branch: bool = False
+
   def get_context_vars(self) -> t.Dict[str, str]:
     return {}
 
@@ -68,19 +85,20 @@ class BaseGitSourceLinker(SourceLinker):
     Compute the URL in the GitHub/Gitea #repo for the API object *obj*.
     """
 
-    if not obj.location:
+    if not obj.location or not obj.location.filename:
       return None
 
     # Compute the path relative to the project root.
-    rel_path = os.path.relpath(os.path.abspath(obj.location.filename), self._project_root)
-    if not nr.fs.issub(rel_path):
+    try:
+      rel_path = Path(obj.location.filename).relative_to(self._project_root)
+    except ValueError:
       logger.debug('Ignored API object %s, path points outside of project root.', obj.name)
       return None
 
     context_vars = self.get_context_vars()
-    context_vars['path'] = rel_path
-    context_vars['sha'] = self._sha
-    context_vars['lineno'] = obj.location.lineno
+    context_vars['path'] = str(rel_path)
+    context_vars['sha'] = (self._branch if self.use_branch else self._sha) or '?'
+    context_vars['lineno'] = str(obj.location.lineno)
 
     url = self.get_url_template().format(**context_vars)
 
@@ -95,13 +113,31 @@ class BaseGitSourceLinker(SourceLinker):
     root and Git SHA at this stage before #get_source_url() is called.
     """
 
+    git = Git(context.directory)
+
     if self.root:
       self._project_root = os.path.join(context.directory, self.root)
     else:
-      self._project_root = _getoutput(['git', 'rev-parse', '--show-toplevel'], cwd=context.directory).strip()
-    self._sha = _getoutput(['git', 'rev-parse', 'HEAD'], cwd=self._project_root).strip()
+      project_root = git.get_toplevel()
+      if not project_root:
+        raise RuntimeError(f'Path "%s" is not in a Git repository', context.directory)
+      self._project_root = project_root
+
+    self._sha = git.rev_parse('HEAD')
+
+    if self.use_branch:
+      try:
+        self._branch: str | None = git.get_current_branch_name()
+        if '(' in self._branch:
+          raise NoCurrentBranchError  # For older versions of nr.util which returned the detached HEAD branch
+      except NoCurrentBranchError:
+        logger.warning('Repository is not currently on a branch, falling back to SHA')
+        self.use_branch = False
+        self._branch = None
+
     logger.debug('project_root = %r', self._project_root)
     logger.debug('sha = %r', self._sha)
+    logger.debug('branch = %r', self._branch)
 
 
 @dataclasses.dataclass
@@ -124,13 +160,8 @@ class GitSourceLinker(BaseGitSourceLinker):
   an API object.
   """
 
-  #: The URL template as a Python format string. Available variables are "path", "sha" and
-  #: "lineno".
-  url_template: str = None
-
-  def __post_init__(self) -> None:
-    # To work around dataclass default order constraints.
-    if not self.url_template: raise ValueError('url_template must be set')
+  #: The URL template as a Python format string. Available variables are "path", "sha" and "lineno".
+  url_template: str
 
   def get_url_template(self) -> str:
     return self.url_template
@@ -158,8 +189,8 @@ class GithubSourceLinker(BaseGitServiceSourceLinker):
 
   def get_url_template(self) -> str:
     return 'https://{host}/{repo}/blob/{sha}/{path}#L{lineno}'
-  
-  
+
+
 @dataclasses.dataclass
 class GitlabSourceLinker(BaseGitServiceSourceLinker):
   """
