@@ -32,6 +32,9 @@ import typing as t
 import docspec
 from databind.core import annotations as A
 
+if t.TYPE_CHECKING:
+  from pydoc_markdown.util.docspec import ApiSuite
+
 
 class Context:
   """
@@ -74,6 +77,16 @@ class Resolver(abc.ABC):
   def resolve_ref(self, scope: docspec.ApiObject, ref: str) -> t.Optional[str]: ...
 
 
+class ResolverV2(abc.ABC):
+  """ New style interface for resolving based on a text ref from in the context of a #docspec.ApiObject
+  to find another. This is different from #Resolver because it returns the resolved object directly, instead
+  of some string representation of it.
+  """
+
+  @abc.abstractmethod
+  def resolve_reference(self, suite: 'ApiSuite', scope: docspec.ApiObject, ref: str) -> t.Optional[docspec.ApiObject]: ...
+
+
 @A.unionclass(A.unionclass.Subtypes.chain(
   A.unionclass.Subtypes.entrypoint('pydoc_markdown.interfaces.Processor'),
   A.unionclass.Subtypes.import_()), style=A.unionclass.Style.flat)
@@ -110,6 +123,25 @@ class Renderer(PluginBase):
   def render(self, modules: t.List[docspec.Module]) -> None: ...
 
 
+class SinglePageRenderer(PluginBase):
+  """
+  Interface for rendering a single page.
+  """
+
+  @abc.abstractmethod
+  def render_single_page(self, fp: t.TextIO, modules: t.List[docspec.Module],
+                         page_title: t.Optional[str] = None) -> None: ...
+
+
+class SingleObjectRenderer(PluginBase):
+  """
+  Interface for rendering a single #docspec.ApiObject.
+  """
+
+  @abc.abstractmethod
+  def render_object(self, fp: t.TextIO, obj: docspec.ApiObject, options: t.Dict[str, t.Any]) -> None: ...
+
+
 class Server(abc.ABC):
   """
   This interface describes an object that can start a server process for
