@@ -27,7 +27,6 @@ With no arguments it will load the default configuration file. If the
 or a YAML formatted object for the configuration.
 """
 
-import dataclasses
 import logging
 import os
 import sys
@@ -95,7 +94,7 @@ class RenderSession:
       if self.search_path:
         loader.search_path = self.search_path
       if self.py2 is not None:
-        loader.print_function = not self.py2
+        loader.parser.print_function = not self.py2
 
     if self.render_toc is not None:
       # Find the #MarkdownRenderer field for this renderer.
@@ -125,7 +124,7 @@ class RenderSession:
       config.init(Context(directory=os.path.dirname(os.path.abspath(self.config))))
 
     if config.unknown_fields:
-      logger.warning('Unknown configuration options: %s', ', '.join(config.unknown_fields))
+      logger.warning('Unknown configuration options:\n%s\n', '\n------\n'.join(config.unknown_fields))
 
     return config
 
